/* QNT - file.cpp - File buffer class / file handling. */


#define WIN32_LEAN_AND_MEAN
#include <io.h>
#include <fstream.h>
#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include <windows.h>
#include "buffer.h"
#include "display.h"
#include "keyboard.h"


void Buffer::setname( const char *fname )
{
    delete filename;
    int i = strlen( fname );
    char *newautosavename = new char[i + 10];
    memcpy( newautosavename, fname, i );
    memcpy( newautosavename + i, ".autosave", 10 );
    if( !access( newautosavename, F_OK ) )
    {
        display.message( "Autosave file seems to exist, overwrite? (Y/n)" );
        if( keyboard.yesnoP() )
        {
            remove( newautosavename );
            if( autosavename )
            {
                rename( autosavename, newautosavename );
                delete autosavename;
            }
            autosavename = newautosavename;
        }
    }
    filename = new char[i + 1];
    memcpy( filename, fname, i + 1 );
}


BOOL Buffer::autosave()
{
    if( autosavename )
        return saveas( autosavename );
    else
        return FALSE;
}


BOOL Buffer::saveas( const char *fname )
{
    int saverow = row;
    int savecol = col;
    filebeg();
    char *tname = tmpnam( NULL );
    ofstream outf;
    outf.open( tname, ios::out | ios::binary );
    int vissize;
    int size;
    char *buf;
    exitline( vissize, size, buf );
    int len;
    while( (len = endlines->getlen()) != 0 )
    {
        outf.write( buf, size );
        startlines = new LineList( vissize, size, buf, startlines );
        vissize = endlines->getvislen();
        size = len;
        buf = endlines->cdrbuf();
        endlines = endlines->cdr();
        row++;
    }
    outf.write( buf, size );
    enterline( vissize, size, buf, 0 );
    seekto( saverow, savecol );
    if( !outf.bad() )
    {
        outf.close();
        remove( fname );
        if( !rename( tname, fname ) )
        {
            modified = changed = FALSE;
            return TRUE;
        }
    }
    remove( tname );
    return FALSE;
}


/***********************************************************************
    User callable:
***********************************************************************/

BOOL Buffer::load()
{
/* File specifics, load the data and be done with it: */
    ifstream inf;
    inf.open( filename, ios::in | ios::binary );
    if( inf.fail() )
        return FALSE;
    inf.seekg( 0, ios::end );
    int len = inf.tellg();
    char *buf = new char[len + 1];
    inf.seekg( 0, ios::beg );
    inf.read( buf, len );
    if( inf.fail() )
        return FALSE;
    inf.close();

/* Buffer specifics: */
    brk();
    build();
    buf[len] = '\n';
    char *ptr = buf;
    rows = 0;
    ent = -1;
    while( ptr < buf + len )
    {
        char *start = ptr;
        while( *ptr != '\n' && *ptr != '\r' ) ptr++;
        int corr = 1;
        int thent;
        if( ptr[0] == '\n' )
        {
            if( ptr < buf + len && ptr[1] == '\r' )
            {
                ptr++;
                corr++;
                thent = 3;
            }
            else
                thent = 0;
        }
        else
        {
            if( ptr[1] == '\n' )
            {
                ptr++;
                corr++;
                thent = 1;
            }
            else
                thent = 2;
        }
        if( ent < 0 )
            ent = thent;
        ptr++;
        char *line = new char[ptr - start];
        memcpy( line, start, ptr - start );
        startlines = new LineList( ptr - start - corr, ptr - start, line, startlines );
        rows++;
        row++;
    }
    delete [] buf;
    filebeg();
    return TRUE;
}


BOOL Buffer::save()
{
    if( filename && saveas( filename ) )
    {
        truemod = FALSE;
        return TRUE;
    }
    else
        return FALSE;
}
